# deduc.py
"""
Déductions automatiques
"""

def deduc_case(grille, ligne, colonne):
    """
    Déductions réalisées sur une Case
    ...
    Paramètre:
    ----------
    grille: Grille
        La grille sur laquelle on fait des déductions
    ligne, colonne: int
        Les coordonnées de la case (de 0 à 9)
    """
    la_case = grille.get_case(ligne, colonne)
    ligne, colonne, secteur = la_case.get_coord()
    une_valeur = la_case.get_value()
    if une_valeur != 0:
        return
    les_values = []

    for une_case in grille.les_lignes[ligne]:
        if une_case == la_case:
            continue
        une_valeur = une_case.get_value()
        if une_valeur != 0:
            les_values.append(une_valeur)

    for une_case in grille.les_colonnes[colonne]:
        if une_case == la_case:
            continue
        une_valeur = une_case.get_value()
        if une_valeur != 0:
            les_values.append(une_valeur)

    for une_case in grille.les_secteurs[secteur]:
        if une_case == la_case:
            continue
        une_valeur = une_case.get_value()
        if une_valeur != 0:
            les_values.append(une_valeur)

    for une_valeur in les_values:
        if une_valeur in la_case.get_candidats():
            grille.remove_candidat(ligne, colonne, une_valeur)

def deduc_type_groupe(grille, type_groupe, ligne, colonne):
    """
    Déductions réalisées sur une Case. On supprime
    les candidats qui apparaissent sur la même ligne (ou colonne
    ou secteur).
    ...
    Paramètre:
    ----------
    grille: Grille
        La grille sur laquelle on fait des déductions
    type_groupe: (str)
        "ligne" ou "colonne" ou "secteur"
    ligne, colonne: (int)
        Les coordonnées de la case (de 0 à 9)
    """
    la_case = grille.get_case(ligne, colonne)
    _, _, secteur = la_case.get_coord()
    if type_groupe == "ligne":
        numero = ligne
    elif type_groupe == "colonne":
        numero = colonne
    elif type_groupe == "secteur":
        numero = secteur
    else:
        return
    les_cases = get_type_groupe(grille, type_groupe, numero)

    une_valeur = la_case.get_value()
    if une_valeur != 0:
        return
    les_values = []

    for une_case in les_cases:
        if une_case == la_case:
            continue
        une_valeur = une_case.get_value()
        if une_valeur != 0:
            les_values.append(une_valeur)

    for une_valeur in les_values:
        if une_valeur in la_case.get_candidats():
            la_case.remove_candidat(une_valeur)

def deduc_ligne_colonne(grille, secteur, value):
    """
    Réalise des déductions basées sur l'algo "LigneColone"
    ...
    Paramètre:
    ----------
    secteur: int
        Le numéro du secteur
    value: int
        La valeur que l'on veut trouver dans le secteur
    """
    contenu_secteur = [0, 0, 0, 0, 0, 0, 0, 0, 0]
    i = 0
    for une_case in grille.les_secteurs[secteur]:
        une_valeur = une_case.get_value()
        if une_valeur != 0:
            if une_valeur == value:
                return
            else:
                contenu_secteur[i] = 1
        i += 1
    lignes = [[0, 1, 2], [0, 1, 2], [0, 1, 2],
              [3, 4, 5], [3, 4, 5], [3, 4, 5],
              [6, 7, 8], [6, 7, 8], [6, 7, 8]]
    colonnes = [[0, 1, 2], [3, 4, 5], [6, 7, 8],
               [0, 1, 2], [3, 4, 5], [6, 7, 8],
               [0, 1, 2], [3, 4, 5], [6, 7, 8]]
    k = 0
    for i in lignes[secteur]:
        for une_case in grille.les_lignes[i]:
            une_valeur = une_case.get_value()
            if une_valeur == value:
                for j in range(3):
                    contenu_secteur[k*3+j] = 1
        k += 1

    k = 0
    for i in colonnes[secteur]:
        for une_case in grille.les_colonnes[i]:
            une_valeur = une_case.get_value()
            if une_valeur == value:
                for j in range(3):
                    contenu_secteur[j*3+k] = 1
        k += 1

    nb_de_zeros = 0
    coord = 0
    for i in range(9):
        if contenu_secteur[i] == 0:
            nb_de_zeros += 1
            coord = i
    if nb_de_zeros != 1:
        return

    i = coord // 3
    j = coord % 3
    ajout = [[0, 0], [0, 3], [0, 6], [3, 0], [3, 3], [3, 6], [6, 0], [6, 3], [6, 6]]
    ligne = i + ajout[secteur][0]
    colonne = j + ajout[secteur][1]
    grille.set_case(ligne, colonne, value)

def get_type_groupe(grille, type_groupe, numero):
    """
    Renvoie un groupe de cases (une ligne, une colonne, un secteur)
    ...
    Paramètres:
    -----------
    grille: Grille
        La grille sur laquelle on fait les déductions
    type_groupe: str
        Le type de groupe ("ligne", "colonne" ou "secteur")
    numero: int
        Le numéro de ligne ou de  colonne  ou de secteur
    """
    if type_groupe == "ligne":
        les_cases = grille.les_lignes[numero]
    elif type_groupe == "colonne":
        les_cases = grille.les_colonnes[numero]
    elif type_groupe == "secteur":
        les_cases = grille.les_secteurs[numero]
    else:
        les_cases = None
    return les_cases

def deduc_multiplet(grille, type_groupe, numero):
    """
    Réalise des déductions basées sur les éliminations des multiplets
    ...
    Paramètre:
    ----------
    type_groupe: str
        Le type de groupe ("ligne", "colonne" ou "secteur")
    numero: int
        Le numéro de ligne ou de  colonne  ou de secteur
    """
    multiplets = {}

    les_cases = get_type_groupe(grille, type_groupe, numero)
    if les_cases is None:
        return

    for une_case in les_cases:
        candidats = une_case.get_candidats()
        cle = str(candidats)
        if cle in multiplets:
            multiplets[cle].append(une_case)
        else:
            multiplets[cle] = [une_case]

    for cle in multiplets.keys():
        objet = eval(cle)
        if len(multiplets[cle]) >= len(objet):
            for une_case in les_cases:
                if une_case in multiplets[cle]:
                    continue
                for un_candidat in objet:
                    une_case.remove_candidat(un_candidat)

def deduc_seule_possible(grille, type_groupe, numero):
    """
    Déductions réalisées sur un groupe (ligne, colonne, secteur)
    On déduit une valeur d'une case si c'est la seule possible pour
    le groupe.
    ...
    Paramètres:
    grille: Grille
        La grille sur laquelle on fait des déductions
    type_groupe: (str)
        "ligne" ou "colonne" ou "secteur"
    numero: int (de 0 à 8)
        Le numéro de la ligne (ou de la colonne ou du secteur) 
    """
    les_cases = get_type_groupe(grille, type_groupe, numero)
    if les_cases is None:
        return

    les_cases_par_valeur = []
    valeurs_trouvees = []

    for une_case in les_cases:
        candidats = une_case.get_candidats()
        if len(candidats) == 1:
            valeurs_trouvees.append( list(candidats)[0] )

    for i in range(10):
        les_cases_par_valeur.append( [] )
    for une_case in les_cases:
        candidats = une_case.get_candidats()
        if len(candidats) == 1: continue
        coord = une_case.get_coord()
        for un_candidat in candidats:
            if un_candidat in valeurs_trouvees:
                continue
            les_cases_par_valeur[un_candidat].append(coord)
    for i in range(1,10):
        if len(les_cases_par_valeur[i]) == 1:
            ligne, colonne, secteur = les_cases_par_valeur[i][0]
            grille.set_case(ligne, colonne, i)



# grille.py
"""
Contient l'API Grille et la classe sous-jacente Case
ainsi que les classes d'exception associées
et des outils vérifie une value ainsi qu'une coordonnée
"""

class CreeGrille(Exception):
    """
    Classe d'exception qui vérifie l'initialisation de la Grille
    """
    def __init__(self, message):
        super().__init__(message)

class ValeurIncorrecte(Exception):
    """
    Classe d'exception qui vérifie si une valeur est valide
    """
    def __init__(self):
        super().__init__("Valeur doit etre un entier valide (dans les bornes")

def is_value_correcte(value, minimum=0, maximum=9):
    """
    Vérifie si value a une valeur correcte (int de minimum à maximum)
    ...
    Paramètres:
    -----------
    value: int
        valeur dont on teste la cohérence (int de minimum à maximum)
    Exceptions:
    -----------
    Lève l'Exception ValeurIncorrecte si valeur est incorrecte
    """
    if not isinstance(value, int):
        print("non entier")
        raise ValeurIncorrecte()
    if value < minimum or value > maximum:
        print("en dehors bornes")
        raise ValeurIncorrecte()

def is_coord_correcte(value):
    """
    Vérifie si value a une valeur correcte (int de 0 à 8)
    ...
    Paramètres:
    -----------
    value: int
        valeur de 0 à 8
    Exceptions:
    - ValeurIncorrecte
    ----------
    Lève l'Exception CoordIncorrecte si valeur incorrecte
    """
    if not isinstance(value, int):
        raise ValueError()
    if value < 0 or value > 8:
        raise ValeurIncorrecte()

class Grille:
    """
    Cette classe représente une grille de Sudoku
    ...
    Attributs:
    ---------
    lst_cases : list de Case
        L'adresse des 81 cases (de type "Case") qui composent une
        Grille.
    les_lignes, les_colonnes, les_secteurs : list de liste de Case
        L'adresse de groupes de cases. Par exemple, les_lignes
        contient l'adresse de 9 groupes, une par ligne. Chaque
        groupe contenant l'adresses des 9 cases formant une ligne.
    log : list de list
        Contient l'historique des opérations effectuées. Ce qui
        permet de faire une opération "undo".
        enreg: "humain"|"ordi",<operation>,ligne,colonne,oldv,newv
        <operation>::= "setc" | "adc" | "rmc"
    debug: bool
        Permet d'activer le mode verbeux (débogage)
    """
    def __init__(self, les_cases: bool, debug: bool=False):
        """
        Constructeur: Crée une Grille
        ...
        Paramètres:
        -----------
        les_cases: list de int
            La valeur initiale d'une Case. La valeur égale à zéro
            signifie une Case vide.
        debug: bool, par défaut False
            Permet d'activer le Debugging
        Exception:
        ---------
        - Lève l'Exception CreeGrille si la Grille est incorrecte
        - Lève l'Exception ValeurIncorrecte si value n'est pas
          entre 0 et 9
        """
        if len(les_cases) != 81:
            raise CreeGrille("Il faut 81 cases")
        for i in range(81):
            is_value_correcte(les_cases[i])

        self.debug = debug
        self.log = []
        self.lst_cases = []
        self.les_lignes = []
        for i in range(9):
            self.les_lignes.append([])
        self.les_colonnes = []
        for i in range(9):
            self.les_colonnes.append([])
        self.les_secteurs = []
        for i in range(9):
            self.les_secteurs.append([])
        ligne = colonne = secteur = 0
        for i in range(81):
            ligne = i // 9
            colonne = i % 9
            coord_x = colonne // 3
            coord_y = ligne // 3
            secteur = coord_y * 3 + coord_x
            une_case = Case(ligne, colonne, secteur, les_cases[i])
            self.lst_cases.append(une_case)
            self.les_lignes[ligne].append(une_case)
            self.les_colonnes[colonne].append(une_case)
            self.les_secteurs[secteur].append(une_case)

    def toggle_debug(self):
        """
        Permute la valeur du mode debug
        """
        if self.debug:
            self.debug = False
        else:
            self.debug = True

    def get_case(self, ligne, colonne):
        """
        Retrouve une Case par ses coordonnées
        ...
        Paramètres:
        ----------
        ligne, colonne: int
            Les coordonnées d'une Case
        Valeur de retour:
        ----------------
        L'adresse de la Case
        ...
        """
        return self.lst_cases[ligne * 9 + colonne]

    def set_synchro(self):
        """
        Démarre une transaction
        """
        self.log.append(["COMMIT"])

    def set_case(self, ligne, colonne, value):
        """
        Modifie la value d'une Case
        ...
        Paramètres:
        -----------
        ligne, colonne, value: int
            ligne, colonne: coordonnées, value: La nouvelle valeur
        """
        une_case = self.get_case(ligne, colonne)
        if une_case.verrou:
            return    # on log ???
        oldvalue = set(une_case.get_candidats())
        if value == 0:
            newvalue = {1,2,3,4,5,6,7,8,9}
            une_case.set_candidats(newvalue)
        else:
            newvalue = {value}
            une_case.set_candidats(newvalue)
        self.log.append(["modif", ligne, colonne, oldvalue])

    def get_value(self, ligne, colonne):
        """
        Retourne la value d'une case
        ...
        Paramètres:
        -----------
        ligne, colonne:
            Coordonnées d'une case
        Valeur de retour:
        -----------------
        La value d'une Case
        """
        une_case = self.get_case(ligne, colonne)
        return une_case.get_value()

    def get_log(self):
        """ Affiche l'historique des modifications
        (Ne fait pas partie de l'API, utilitaire de Debug)
        """
        print("debug: le journal des modifications")
        for item in self.log:
            print(item)

    def get_nb_connu(self):
        """ Renvoie le nombre de Cases occupées """
        nbconnu = 0
        for une_case in self.lst_cases:
            if une_case.get_value() != 0:
                nbconnu += 1
        return nbconnu

    def get_nb_candidats(self):
        """ Renvoie le nombre de Candidats restants """
        nbcandidats = 0
        for une_case in self.lst_cases:
            nbcandidats += len(une_case.get_candidats())
        return nbcandidats

    def get_value_connu(self):
        """ renvoie le tableau des value connus
        Exemple: si on a trouvé tous les "8", alors le "8"
        fait partie des value connus.
        """
        nb_par_value = [0]*10
        for une_case in self.lst_cases:
            value = une_case.get_value()
            if value != 0:
                nb_par_value[value] += 1
        value_trouvees = []
        for i in range(1,10):
            if nb_par_value[i] == 9:
                value_trouvees.append(i)
        return value_trouvees

    def verif(self):
        """ Vérifie si il y a des incohérences, par exemple
        deux 7 dans la même ligne
        ...
        Valeur de retour: True si le test réussie
        """
        verif_ok = True
        # 1. On vérifie les lignes
        for i in range(9):
            les_valeurs = [0]*10
            for j in range(9):
                index = self.les_lignes[i][j].get_value()
                if index == 0:
                    continue
                if les_valeurs[index] != 0:
                    verif_ok = False
                    if self.debug:
                        print("debug: Verif Ligne ECHOUE")
                else:
                    les_valeurs[index] = 1
        # 2. On vérifie les colonnes
        for i in range(9):
            les_valeurs = [0]*10
            for j in range(9):
                index = self.les_colonnes[i][j].get_value()
                if index == 0:
                    continue
                if les_valeurs[index] != 0:
                    verif_ok = False
                    if self.debug:
                        print("debug: Verif Colone ECHOUE")
                else: les_valeurs[index] = 1
        # 3. On vérifie les secteurs
        for i in range(9):
            les_valeurs = [0]*10
            for j in range(9):
                index = self.les_secteurs[i][j].get_value()
                if index == 0:
                    continue
                if les_valeurs[index] != 0:
                    verif_ok = False
                    if self.debug:
                        print("debug: Verif Secteur ECHOUE")
                else: les_valeurs[index] = 1
        return verif_ok

    def undo(self, snapshot="COMMIT"):
        """
        Annule la dernière action (revient au dernier point de synchro)
        ...
        Paramètres:
        -----------
        snapshot: str
            Jusqu'où on revient en arrière, par défaut la dernière opération.
        """
        while True:
            if len(self.log) >= 1:
                enreg = self.log.pop()
                if enreg[0] == snapshot:
                    break
                if self.debug:
                    print(">>>UNDO: ", enreg)
                else:
                    une_case = self.get_case(enreg[1], enreg[2])
                    une_case.set_candidats(enreg[3])
            else:
                break

    def add_candidat(self, ligne, colonne, value):
        """
        Ajoute un candidat (à priori, provient d'une action "UNDO")
        ...
        Paramètres:
        -----------
        ligne, colonne, value: int
            ligne, colonne: coordonnées, value: La nouvelle valeur
        """
        une_case = self.get_case(ligne, colonne)
        if une_case.verrou:
            return    # on log ???
        oldvalue = set(une_case.get_candidats())
        une_case.add_candidat(value)
        self.log.append(["modif", ligne, colonne, oldvalue])

    def remove_candidat(self, ligne, colonne, value):
        """
        Supprime un candidat (à priori, c'est la base des déductions
        ...
        Paramètres:
        -----------
        ligne, colonne, value: int
            ligne, colonne: coordonnées, value: La nouvelle valeur
        """
        une_case = self.get_case(ligne, colonne)
        if une_case.verrou:
            return    # on log ???
        oldvalue = set(une_case.get_candidats())
        une_case.remove_candidat(value)
        self.log.append(["modif", ligne, colonne, oldvalue])

    def get_candidats(self, ligne, colonne):
        """
        Renvoie la liste des candidats d'une Case
        ...
        Paramètres:
        -----------
        ligne, colonne: int
            Les coordonnées de la case
        Valeur de retour:
        -----------------
        Les candidats de la case
        """
        une_case = self.get_case(ligne, colonne)
        return une_case.get_candidats()

class Case:
    """
    Cette classe représente une Case d'une Grille de Sudoku
    ...
    ATTRIBUTS:
    ----------
    ligne, colonne, secteur: int
        Les coordonnées de la Case
    value: int
        Le contenu de la Case. La valeur 0 signifie une Case vide
    verrou: bool
        Si vrai, la Case ne peut pas être modifié
        Elle appartient au problème, ce ne peut être une hypothèse
    candidats: list de int
        Les candidats, ie les valeurs possible de la Case.
        Si il ne reste qu'un seul candidat, celui-ci affecte la value

    """
    def __init__(self, ligne, colonne, secteur, value):
        """
        constructeur
        ...
        Paramètres:
        -----------
        ligne, colonne, secteur, value: int
            Respectivement: ligne, colonne, secteur, value
        """
        self.ligne = ligne
        self.colonne = colonne
        self.secteur = secteur
        self.value = value
        self.candidats = {i for i in range(1, 10)}
        if value > 0:
            self.verrou = True # ce doit etre une methode a part!!!
            self.candidats = {value}
        else:
            self.verrou = False

    def get_value(self):
        """
        Renvoie la value d'une Case
        ...
        Valeur de retour: la value d'une Case (int)
        """
        return self.value

    def add_candidat(self, value):
        """
        Ajoute un candidat à une Case (souvent suite à un "UNDO")
        (éventuellement supprime la value [la remet à zéro] si
        maitenant on a plusieurs [2] candidats).
        ...
        Paramètres:
        -----------
        value: int
            La valeur du candidat que l'on ajoute
        """
        if value not in self.candidats:
            self.candidats.add(value)
            if len(self.candidats) == 2:
                self.value = 0

    def remove_candidat(self, value):
        """
        Retirer un candidat (l'opération de base d'une déduction)
        ...
        Paramètres:
        -----------
        value: int
            La valeur du candidat que l'on retire
        """
        if len(self.candidats) == 1:
            return
        if value in self.candidats:
            self.candidats.remove(value)
        if len(self.candidats) == 1:
            self.value = list(self.candidats)[0]

    def get_candidats(self):
        """
        Renvoie les candidats d'une Case
        ...
        Valeur de retour: les candidats (list d'int)
        """
        return self.candidats

    def get_coord(self):
        """
        Renvoie les coordonnées d'une Case
        ...
        Valeur de retour:
        -----------------
        ligne, colonne, secteur: int
            Les coordonnées de la Case
        """
        return self.ligne, self.colonne, self.secteur

    def set_candidats(self, new_candidats):
        """
        Fixe l'ensemble des candidats
        ...
        Paramètres:
        -----------
        new_candidats: set
            Les nouveaux candidats
        """
        self.candidats = new_candidats
        if len(self.candidats) == 1:
            self.value = list(self.candidats)[0]
        else:
            self.value = 0



# __main__.py : le programme principal
"""
L'interface texte (TUI) à la bibliothèque grille qui permet
de jouer au SUDOKU
"""
# V2.0.0 (objectif: V2.0)
import sys
import re
import string
from .grille import Grille, is_value_correcte, CreeGrille
from .deduc import deduc_case, deduc_type_groupe, deduc_ligne_colonne, deduc_multiplet, deduc_seule_possible, get_type_groupe

def aide():
    """
    Affiche l'aide en ligne
    """
    print("quit:    quitter")
    print("help:    aide")
    print("tbug:    Permute le mode DEBUG")
    print("veri:    vérification de la cohérence de la grille")
    print("undo:    on annule la dernière opération")
    print("addc:    Ajoute un(des) candidat(s) à une case, (addc,F4,34)")
    print("rmvc:    Supprime un(des) candidat(s) d'une case, (rmvc,F4,7)")
    print("getc:    Liste les candidats d'une case, (getc,F4)")
    print("dedu:    Réaliser des déductions automatiques")
    print("glog:    Affiche le journal des modifications")
    print("gcan:    Affiche tous les candidats d'un groupe (gcan,<type>,4)")
    print("         Remarque: les types:  ligne | colonne | secteur")

def nouvelle_partie(un_fichier):
    """
    Lit et décode un fichier contenant une Grille
    ...
    Paramètres:
    -----------
    un_fichier: str
        Le chemin du fichier

    Valeur de retour:
    -----------------
    Une instance de Grille

    Exceptions:
    -----------
    - FileNotFoundError
    - Exception si la Grille a un format incorrecte
    """
    handler = open(un_fichier, "r")
    chaine = handler.read()
    lst_cases = []
    for car in chaine:
        if   car == ".":
            lst_cases.append(0)
        elif car.isnumeric():
            lst_cases.append(int(car))
    instance = Grille(lst_cases)
    handler.close()
    return instance

def affiche_grille():
    """
    Affiche la Grille
    """
    value_trouvees = LA_GRILLE.get_value_connu()
    print("    A B C |D E F |G H I", end="")
    if LA_GRILLE.debug:
        print(" (mode DEBUG actif)")
    else: print()
    for i in range(9):
        if (i % 3) == 0:
            print("   ---------------------")
        print("%2d " % (i+1), end="")
        for j in range(9):
            if (j % 3) == 0:
                print("|", end="")
            contenu = LA_GRILLE.get_case(i, j).get_value()
            if contenu == 0:
                contenu = "."
            else:
                contenu = str(contenu)
            print(contenu, end=" ")
        if (i+1) in value_trouvees:
            print("*",end="")
        print()
    print("   ---------------------")
    nbconnu = LA_GRILLE.get_nb_connu()
    nbcandidats = LA_GRILLE.get_nb_candidats()
    print("Nombre de cases connues: %d, candidats: %d" % (nbconnu, nbcandidats))

def gcan(type_groupe, numero):
    """ affiche les candidats d'un groupe """
    les_cases = get_type_groupe(LA_GRILLE, type_groupe, numero)
    if les_cases is None:
        return
    for une_case in les_cases:
        ligne, colonne, _ = une_case.get_coord()
        les_candidats = une_case.get_candidats()
        if len(les_candidats) == 1:
            continue
        print(", [%s%d]: " % (COLONNES[colonne], ligne+1), end="")
        print(les_candidats, end=" ")
    print()

def  deduction():
    """
    Sous-menu associé aux déductions
    """
    old_cmd = "quit"
    LA_GRILLE.set_synchro()
    while True:
        affiche_grille()
        print("===== deductions pour une case ou une ligne, ...  ======")
        print("type:        déduction par ligne/colonne/secteur (type,<type>,F4)")
        print("             Remarque: les types: ligne | colonne | secteur")
        print("case:        déduction pour une case (case,F4)")
        print("multiplet:   (multiplet,<type>,5)")
        print("seule:       Seule possibilité, (seule,<type>,6)") 
        print("ligcol:      Algo Ligne/Colonne, (ligcol,4,5) [secteur,valeur]")
        print("===== deductions évoluées ========")
        print("cases:       idem de candidtat, mais pour toutes les cases")
        print("ligscols:    Idemd de ligcol, mais pour tous les secteurs et valeurs")
        print("seules:      Seule possibilité, mais pour tous les groupes") 
        print("multiplets:  Multiplet pour toutes les cases")
        print("----------------------------------")
        print("TOUT:        Utilise toutes les méthodes")
        print("quit:        QUITTER")
        choix = input("Commande ? ")
        if  choix == "":
            choix = old_cmd
        else:
            old_cmd = choix
        if choix == "quit":
            break
        elif choix[:4] == "type":
            match = re.search(r'^type,(ligne|colonne|secteur),([A-I])([1-9])', choix)
            if match:
                type_groupe = match.group(1)
                colonne = COLONNES.find(match.group(2))
                ligne = int(match.group(3))-1
                LA_GRILLE.set_synchro()
                deduc_type_groupe(LA_GRILLE, type_groupe, ligne, colonne)
        elif choix[:10] == "multiplet,":
            match = re.search('^multiplet,(ligne|colonne|secteur),([1-9])', choix)
            if match:
                type_groupe = match.group(1)
                numero = int(match.group(2))-1
                LA_GRILLE.set_synchro()
                deduc_multiplet(LA_GRILLE, type_groupe, numero)
        elif choix[:5] == "case,":
            match = re.search(r'^case,([A-I])([1-9])', choix)
            if match:
                colonne = COLONNES.find(match.group(1))
                ligne = int(match.group(2))-1
                print("===case===>", ligne, colonne)
                deduc_case(LA_GRILLE, ligne, colonne)
        elif choix[:6] == "seule,":
            match = re.search('^seule,(ligne|colonne|secteur),([1-9])', choix)
            if match:
                type_groupe = match.group(1)
                numero = int(match.group(2))-1
                LA_GRILLE.set_synchro()
                deduc_seule_possible(LA_GRILLE, type_groupe, numero)
        elif choix == "cases":
            for i in range(9):
                for j in range(9):
                    deduc_case(LA_GRILLE, i, j)
        elif choix == "multiplets":
            for type_groupe in ["ligne", "colonne", "secteur"]:
                for i in range(9):
                    deduc_multiplet(LA_GRILLE, type_groupe, i)
        elif choix == "seules":
            for type_groupe in ["ligne", "colonne", "secteur"]:
                for i in range(9):
                    deduc_seule_possible(LA_GRILLE, type_groupe, i)
        elif choix[:7] == "ligcol,":
            match = re.search(r'ligcol,([1-9]),([1-9])')
            if match:
                secteur = match.group(1) - 1
                valeur = match.group(2)
                deduc_ligne_colonne(LA_GRILLE, secteur, valeur)
        elif choix == "ligscols":
            for secteur in range(9):
                for valeur in range(1, 10):
                    deduc_ligne_colonne(LA_GRILLE, secteur, valeur)
        elif choix == "TOUT":
            nbcandidats = LA_GRILLE.get_nb_candidats()
            while True:
                for secteur in range(9):
                    for valeur in range(1, 10):
                        deduc_ligne_colonne(LA_GRILLE, secteur, valeur)
                for i in range(9):
                    for j in range(9):
                        deduc_case(LA_GRILLE, i, j)
                for type_groupe in ["ligne", "colonne", "secteur"]:
                    for i in range(9):
                        deduc_multiplet(LA_GRILLE, type_groupe, i)
                for type_groupe in ["ligne", "colonne", "secteur"]:
                    for i in range(9):
                        deduc_seule_possible(LA_GRILLE, type_groupe, i)
                nbcandidatsnew = LA_GRILLE.get_nb_candidats()
                if nbcandidatsnew < nbcandidats:
                    nbcandidats = LA_GRILLE.get_nb_candidats()
                    continue
                break

def grande_boucle():
    """
    boucle principale du programme. A chaque tour de boucle
    le joueur saisie une hypothèse pour une case ou active
    une des commandes du menu.
    """
    while True:
        affiche_grille()
        saisie = input("Remplir une case: 'F4=4' (saisie vide ou 'help': menu) ? ")
        if saisie == "quit":
            break
        elif saisie == "help" or saisie == "":
            aide()
        elif "=" in saisie:
            print(">>>> Fixe la valeur d'une Case (0: Annule une saisie)")
            match = re.search(r'([A-I])([1-9])=([0-9])', saisie)
            if match:
                ligne = int(match.group(2))-1
                value = int(match.group(3))
                colonne = COLONNES.find(match.group(1))
                LA_GRILLE.set_synchro()
                LA_GRILLE.set_case(ligne, colonne, value)
        elif saisie == "tbug":
            LA_GRILLE.toggle_debug()
        elif saisie[:4] == "gcan":
            match = re.search(r'^gcan,(secteur|colonne|ligne),([1-9])', saisie)
            if match:
                gcan(match.group(1),int(match.group(2))-1)
        elif saisie == "glog":
            LA_GRILLE.get_log()
        elif saisie == "veri":
            if LA_GRILLE.verif():
                print("========> La grille est correcte")
            else:
                print("ERREUR (2 valeurs identiques sur une ligne, colonne, secteur")
        elif saisie == "undo":
            print(">>>> Annule la dernière opération")
            LA_GRILLE.undo()
        elif saisie[:4] == "addc":
            print(">>>> Ajoute un candidat à une Case")
            match = re.search(r'^addc,([A-I])([1-9]),([1-9]+)', saisie)
            if match:
                ligne = int(match.group(2))-1
                colonne = COLONNES.find(match.group(1))
                LA_GRILLE.set_synchro()
                for value in match.group(3):
                    LA_GRILLE.add_candidat(ligne, colonne, int(value))
        elif saisie[:4] == "rmvc":
            print(">>>> Retire un candidat à une Case")
            match = re.search(r'^rmvc,([A-I])([1-9]),([1-9]+)', saisie)
            if match:
                ligne = int(match.group(2))-1
                colonne = COLONNES.find(match.group(1))
                LA_GRILLE.set_synchro()
                for value in match.group(3):
                    LA_GRILLE.remove_candidat(ligne, colonne, int(value))
        elif saisie[:4] == "getc":
            print(">>>> Affiche les candidats d'une Case")
            match = re.search(r'^getc,([A-I])([1-9])', saisie)
            if match:
                ligne = int(match.group(2))-1
                colonne = COLONNES.find(match.group(1))
                print(LA_GRILLE.get_candidats(ligne, colonne))
        elif saisie == "dedu":
            deduction()

    nbconnu = LA_GRILLE.get_nb_connu()
    verification = LA_GRILLE.verif()
    if nbconnu == 81 and verification:
        print("Vous avez réussi, BRAVO!!!")
    return

#========== DEBUT DU PROGRAMME PRINCIPAL ==========================
if len(sys.argv) < 2:
    print(f"Syntaxe: {sys.argv[0]} une_grille")
    print("  Le fichier qui contient une grille est fait de 9 lignes")
    print("  Exemple de ligne:   ..65..7...")
    sys.exit(1)
FICHIER = sys.argv[1]
try:
    LA_GRILLE = nouvelle_partie(FICHIER)
except FileNotFoundError:
    print("ERREUR: le fichier n'existe pas")
    sys.exit(2)
except PermissionError:
    print("ERREUR: vous n'avez pas les droits d'accès au fichier")
    sys.exit(2)
except CreeGrille:
    print("ERREUR: le format du fichier est incorrect")
    sys.exit(2)
#------------------------------------
COLONNES = "ABCDEFGHI"
grande_boucle()



import os
from setuptools import setup, find_packages

setup(
    name="sudokujf",
    version="2.3",
    author="Jean-François BOUCHAUDY",
    author_email="jfbouch@wanadoo.fr",
    description=("Un programme en mode texte pour jouer au SUDOKU"),
    keywords="sudoku tui deduction",
    url="http://linux.home/PyPI/sudokujf",
    packages=find_packages(),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    include_package_data=True,
    licence="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[] 
)
